using System;
using System.Threading;

public class Worker
{
    private int size;
    private int count;
    private double[,] a;
    
    public Worker(int _size, int _count)
    {
        int i, j;
        size = _size;
        count = _count;
        a = new double[size,size];
        for(i=0; i<size; i++)
            for(j=0; j<size; j++)
                a[i,j] = i%7 + j%13;
    }
    
    public void Work()
    {
        int i;
        double[] w;
        double[,] u, vt;
        
        for(i=0; i<count; i++)
            alglib.rmatrixsvd(a, size, size, 2, 2, 2, out w, out u, out vt);
    }
};

public class Simple
{
    public static int Main()
    {
        int nthreads = 1;       // number of worker threads
        int size = 16;          // matrix size
        int count = 512*8*8;    // number of total factorization made (for all threads in total)
        int i;
        Thread[] WorkerThreads = new Thread[nthreads];
        
        // power up JIT compiler
        {
            Worker w = new Worker(5, 5);
            Thread tmpthread = new Thread(new ThreadStart(w.Work));
            tmpthread.Start();
            tmpthread.Join();
        }
        
        // test
        Console.WriteLine("Test");
        for(i=0; i<nthreads; i++)
        {
            Worker w = new Worker(size, count/nthreads);
            WorkerThreads[i] = new Thread(new ThreadStart(w.Work));
        }
        for(i=0; i<nthreads; i++)
            WorkerThreads[i].Start();
            
        System.Diagnostics.Stopwatch sw = System.Diagnostics.Stopwatch.StartNew();
        for(i=0; i<nthreads; i++)
            WorkerThreads[i].Join();
        sw.Stop();
        System.Console.Write("Passed ");
        System.Console.Write(sw.ElapsedMilliseconds);
        System.Console.WriteLine(" ms");
        System.Console.WriteLine("Done!");
        return 0;
    }
}